import base64
import hashlib
from Crypto.Cipher import AES
from Crypto.Util.Padding import pad, unpad

from .file import check_path_is_exits


def to_encode(connect: str) -> str:
    return str(
        base64.b64encode(bytes(connect, encoding="utf8")),
        encoding="utf8",
    )


def to_decode(connect: str) -> str:
    return str(base64.b64decode(bytes(connect, encoding="utf8")), encoding="utf8")


def sum_md5(param_str: str) -> str:
    """计算字符串MD5值"""
    if isinstance(param_str, int):
        param_str = str(param_str)
    _hash = hashlib.md5()
    _hash.update(param_str.encode("UTF-8"))
    return _hash.hexdigest()


def md5sum(*, _file_path: str = None, _string: str = None):
    """计算文件md5值"""
    if _file_path:
        check_path_is_exits(_file_path, path_type="file")
        with open(_file_path, "rb") as _file:
            return hashlib.md5(_file.read()).hexdigest()

    if _string:
        m = hashlib.md5()
        m.update(_string.encode())
        return m.hexdigest()

    raise ValueError("参数_file_path与_string必须二选一，不能都为空")


class MyCrypto(object):
    @classmethod
    def encrypt(cls, data, xsrf_token=None, old_secret=None) -> str:
        """加密"""
        if xsrf_token:
            key = cls.sha256(xsrf_token)
        else:
            key = old_secret
        if len(key) >= 32:
            key = key[:32]
        else:
            key = cls.__add_to_32(key)
        cipher1 = AES.new(key=key.encode("utf-8"), mode=AES.MODE_ECB)
        ct = cipher1.encrypt(pad(data.encode("utf-8"), 16))
        encrypt_data = base64.b64encode(ct)
        return encrypt_data.decode("utf-8")

    @classmethod
    def decrypt(cls, data, xsrf_token=None, old_secret=None) -> str:
        """解密"""
        if xsrf_token:
            key = cls.sha256(xsrf_token)
        else:
            key = old_secret
        if len(key) >= 32:
            key = key[:32]
        else:
            key = cls.__add_to_32(key)
        ct = base64.b64decode(data)
        cipher2 = AES.new(key=key.encode("utf-8"), mode=AES.MODE_ECB)
        pt = unpad(cipher2.decrypt(ct), 16)
        return pt.decode("utf-8")

    @classmethod
    def __add_to_32(cls, text):
        while len(text) % 32 != 0:
            text += "\0"
        return text

    @classmethod
    def sha256(cls, data):
        """sha256加密"""
        return hashlib.sha256(data.encode("utf-8")).hexdigest()

    @staticmethod
    def aes_cbc_encrypt(key, content, iv):
        """AES CBC加密"""

        def __pad(text):
            bs = 16
            padding_size = len(text.encode("utf-8"))
            padding = bs - padding_size % bs
            padding_text = chr(padding) * padding
            return text + padding_text

        cipher = AES.new(key.encode("utf-8"), AES.MODE_CBC, iv.encode("utf-8"))
        # 处理明文
        content_padding = __pad(content)
        # 加密
        encrypt_bytes = cipher.encrypt(content_padding.encode("utf-8"))
        # 重新编码
        result = str(base64.b64encode(encrypt_bytes), encoding="utf-8")
        return result

    @staticmethod
    def aes_cbc_decrypt(key, content, iv):
        """AES CBC解密"""

        def __un_pad(_s):
            return _s[: -ord(_s[len(_s) - 1 :])]

        cipher = AES.new(key.encode("utf-8"), AES.MODE_CBC, iv.encode("utf-8"))
        content = base64.b64decode(content)
        text = cipher.decrypt(content).decode("utf-8")
        return __un_pad(text)



#!/usr/bin/env python
# coding:utf-8
"""redis缓存
作者：dengqingyong
邮箱：yu12377@163.com
时间：2020-06-28
"""
import os
import sys
import json
from pathlib import Path
from typing import Union
from redis import Redis, ConnectionPool

from . import logger
from .utils import byte_to_str, get_caller_info
from .encrypt import to_decode, sum_md5
from .file import read_file_content, save_json_to_file


def get_cache_obj(db_index: int = 15):
    """获取缓存对象
    :param db_index: int, redis中数据库索引编号
    """
    if os.environ.get("REDIS_DB_CONNECT"):
        try:
            _redis = MyRedis(db_index)
        except Exception as _e:
            pass
        else:
            logger.debug(f"redis缓存初始成功！")
            return _redis

    if sys.platform == "win32":
        _target_path = Path(os.environ["USERPROFILE"])
    else:
        _target_path = Path(os.environ["HOME"])
    __cache_path__ = _target_path / "MySession.json"
    logger.warning(f"redis缓存不可用，启动本地文件缓存：{__cache_path__}")
    return MyCache(__cache_path__)


def get_redis_pool(db_index=0, max_connections=None):
    """获取redis连接池
    :param db_index: int, redis中数据库索引编号
    :param max_connections: int, 最大连接数
    """
    _connect = json.loads(to_decode(os.environ.get("REDIS_DB_CONNECT")))
    return ConnectionPool(
        host=_connect["host"],
        port=_connect["port"],
        password=_connect["pass"],
        db=db_index,
        max_connections=max_connections,
    )


def get_redis_handler(db_index=0):
    """获取redis连接
    :param db_index: int, redis中数据库索引编号
    """
    try:
        redis_obj = Redis(connection_pool=get_redis_pool(db_index))
        if redis_obj.ping():
            return redis_obj
        else:
            return None
    except Exception:
        return None


def batch_delete_key(db_index: int, name_expression: str):
    """批量删除redis key
    :param db_index: int, redis中数据库索引编号
    :param name_expression: str, key值通配符
    """
    redis = get_redis_handler(db_index)
    temp_lock_list = redis.keys(name_expression)
    if len(temp_lock_list) > 0:
        redis.delete(*temp_lock_list)


class MyRedis(Redis):
    def __init__(self, db_index=0):
        super().__init__(connection_pool=get_redis_pool(db_index))

    def batch_delete(self, name_expression):
        key_list = self.keys(name_expression)
        if len(key_list) > 0:
            self.delete(*key_list)

    def get_to_json(self, name):
        if name:
            value_raw = self.get(name)
            if value_raw:
                try:
                    user = json.loads(value_raw)
                except (TypeError, json.decoder.JSONDecodeError) as e:
                    self.delete(name)
                else:
                    return MyDict(**user)
        return None

    def set_obj(self, prefix: str, value, **kwargs):
        if isinstance(value, (dict, list, tuple)):
            value = json.dumps(value, ensure_ascii=False)
            key = prefix + sum_md5(value)
            self.set(key, value, **kwargs)
            return key
        else:
            raise TypeError("此方法只用于保存对象！")

    def push_obj(self, queue_name: str, value):
        if isinstance(value, (dict, list, tuple)):
            value = json.dumps(value, ensure_ascii=False)
            return self.rpush(queue_name, value)
        else:
            return self.rpush(queue_name, value)

    def pop_obj(self, queue_name: str):
        value = self.lpop(queue_name)
        try:
            return json.loads(value)
        except Exception as e:
            return value

    def hgetall(self, _k):
        _v = super(MyRedis, self).hgetall(_k)
        if isinstance(_v, dict):
            return {
                _k.decode("utf-8"): _v.decode("utf-8")
                for _k, _v in _v.items()
            }
        else:
            return _v

    def lpop(self, name):
        _k = super(MyRedis, self).lpop(name)
        if _k:
            return _k.decode("utf-8")
        else:
            return _k

    def keys(self, pattern='*'):
        _l = super(MyRedis, self).keys(pattern)
        if _l:
            return [_i.decode("utf-8") for _i in _l]
        else:
            return _l

    def get(self, name):
        _v = super(MyRedis, self).get(name)
        if _v:
            _v = _v.decode("utf-8")
        return _v

    def smembers(self, name):
        _l = super(MyRedis, self).smembers(name)
        if _l:
            return [_i.decode("utf-8") for _i in _l]
        else:
            return _l


class MyCache(object):
    """自定义缓存类，redis不可用时替代使用"""

    def __init__(self, cache_path: Union[Path, str]):
        self._dict = dict()
        self.cache_path = Path(cache_path)
        if self.cache_path.exists():
            cache = read_file_content(self.cache_path, encoding="utf-8", _return="json")
            if isinstance(cache, dict):
                self._dict.update(cache)
            else:
                raise ValueError(f"缓存文件内容格式错误，无法转换成python dict对象：{self.cache_path}")
        else:
            self.cache_path.touch()

    def save_cache(self):
        save_json_to_file(self._dict, self.cache_path)

    def exists(self, key: str):
        return key in self._dict

    def get(self, key: str):
        return self._dict.get(key)

    def delete(self, key: str):
        if key in self._dict:
            self._dict.pop(key)
            self.save_cache()

    def set(self, key: str, value):
        self._dict[key] = value
        self.save_cache()

    def setex(self, name, time, value):
        self._dict[name] = value
        self.save_cache()

    def hmset(self, key: str, value):
        self.set(key, value)

    def hgetall(self, key: str):
        return self.get(key)

    def hset(self, name, key=None, value=None, mapping=None):
        if key is None and not mapping:
            raise ValueError("'key'或'mapping'不能同时为空")

        items = dict()
        if key is not None:
            items[key] = value
        if mapping:
            items.update(mapping)
        self._dict[name] = items
        self.save_cache()

    def __getattr__(self, item):
        logger.warning(f"本地文件类缓存没有实现{item}方法，忽略")

        def __temp_func(*args, **kwargs):
            pass

        return __temp_func


class MyDict(dict):
    """自定义的字典类"""

    def __init__(self, *args, **kwargs):
        super(MyDict, self).__init__(*args, **kwargs)

    def __getattr__(self, item):
        return self[item]

    def __delattr__(self, item):
        del self[item]

    def __setattr__(self, key, value):
        self[key] = value


def hget(name, key, default=None, _type=None, db_index=1):
    if _type not in (None, int, float, str, bool, bytes):
        raise TypeError("_type只能是None,int,float,str,bool,bytes之一")

    redis_obj = MyRedis(db_index)
    value = redis_obj.hget(name, key)

    if value is None:
        return default

    if value and _type == bytes:
        return value

    value = byte_to_str(value)
    if _type:
        if _type == bool:
            if str(value).lower() in ("1", "true", "yes"):
                return True
            else:
                return False
        else:
            return _type(value)
    return value



import os
import re
import json
import time
import stat
import shutil
import tarfile
import zipfile
import logging
import datetime
import configparser
from pathlib import Path
from typing import Union


logger = logging.getLogger("DengUtils")


def check_path_is_exits(src_path: Union[str, Path], path_type=None):
    """检查目录或文件是否存在
    :param src_path: 源路径
    :param path_type: 源路径类型，可选值：file/dir
    """
    if isinstance(src_path, str):
        src_path = Path(src_path)

    if not src_path.exists():
        raise FileNotFoundError(f"目录或文件不存在：{src_path}")

    if path_type and path_type.lower() not in ("file", "dir"):
        raise ValueError("参数path_type非法，可选值有：file/dir")

    if path_type and path_type.lower() == "file":
        if not src_path.is_file():
            raise FileNotFoundError(f"{src_path}不是有效的文件！")

    if path_type and path_type.lower() == "dir":
        if not src_path.is_dir():
            raise FileNotFoundError(f"{src_path}不是有效的目录！")


def ensure_empty_dir(
    target: Union[str, Path], mkdir=True, parents=True, *args, **kwargs
) -> None:
    """确保目录为空目录
    :param target: 目标目录路径
    :param mkdir: bool, 目标目录不存在时自动创建
    :param parents: bool, 是否递归创建目录
    """
    if isinstance(target, str):
        target = Path(target)

    # 删除目录时，时常报拒绝访问的错误，增加重试次数
    for _ in range(10):
        try:
            if target.exists():
                if target.is_dir() and len(os.listdir(str(target))) == 0:
                    logger.info(f"已经是空目录：{target}")
                else:
                    if target.is_dir():
                        shutil.rmtree(target, ignore_errors=False)
                        logger.info(f"删除非空目录成功：{target}")
                    else:
                        target.unlink()
                        logger.info(f"删除文件成功：{target}")
                    # 防止报错，有时rmtree删除命令返回成功，但目录并还没有被删除完成，直接创建目录会报错
                    time.sleep(2)
                    if mkdir:
                        target.mkdir(parents=parents)
                        logger.info(f"创建目录成功：{target}")
            else:
                if mkdir:
                    target.mkdir(parents=parents)
                    logger.info(f"创建目录成功：{target}")
        except Exception as e:
            logger.exception(e)
            time.sleep(1)
        else:
            break
    else:
        raise ValueError(f"删除目录出错：{target}")


def compress_zip(src_path: str, compress_abs_path: str) -> None:
    """压缩zip文件
    :param src_path: 待压缩文件或目录路径
    :param compress_abs_path: 压缩包输出绝对路径
    """
    if not os.path.exists(src_path):
        raise FileNotFoundError(src_path)

    with zipfile.ZipFile(compress_abs_path, "w", zipfile.ZIP_DEFLATED) as compress_file:
        if os.path.isdir(src_path):
            for abs_dir_path, dir_list, file_list in os.walk(src_path):
                relative_path = abs_dir_path.replace(src_path, "")
                relative_path = (relative_path and relative_path + os.sep) or ""
                for filename in file_list:
                    compress_file.write(
                        os.path.join(abs_dir_path, filename), relative_path + filename
                    )
        else:
            filename = os.path.basename(src_path)
            compress_file.write(src_path, filename)


def extract_zip(src_zip: Union[str, Path], dst_dir: Union[str, Path]) -> None:
    """解压zip文件
    :param src_zip: str or Path, 需要解压的zip文件绝对路径
    :param dst_dir: str or Path, 解压后存储的目标目录
    """
    if isinstance(src_zip, str):
        src_zip = Path(src_zip)
    if isinstance(dst_dir, str):
        dst_dir = Path(dst_dir)

    # zip文件不存在时报错
    if not src_zip.exists():
        raise ValueError(f"zip文件不存在：{src_zip}")

    # 解压目录不存在时新建
    if not dst_dir.exists():
        dst_dir.mkdir(parents=True)

    if dst_dir.is_file():
        raise ValueError(f"{dst_dir}不是一个有效的目录")

    if zipfile.is_zipfile(src_zip):
        fz = zipfile.ZipFile(src_zip, "r")
        for file in fz.namelist():
            fz.extract(file, dst_dir)
    else:
        raise ValueError(f"{src_zip}不是一个有效的zip文件")


def compress_tgz(source_files: str, compress_name: str):
    """将源文件打包成tar.gz格式
    :param source_files：str, 源文件路径，传入相对路径时，压缩包中也为相当路径，为绝对路径时，压缩包中同样为绝对路径
    :param compress_name: str, 生成的压缩包路径
    """
    # 判断文件是否存在，不存在时抛错
    if not os.path.exists(source_files):
        raise FileNotFoundError(source_files)

    with tarfile.open(compress_name, "w:gz") as tar:
        if os.path.isdir(source_files):
            for root, _dir, files in os.walk(source_files):
                for file in files:
                    fullpath = os.path.join(root, file)
                    tar.add(fullpath)
        else:
            tar.add(source_files)


def uncompress_tgz(compress_file, target_path="."):
    """解压tar.gz格式文件
    :param compress_file: str, 压缩包路径
    :param target_path: str, 解压后存储路径
    """
    tar = tarfile.open(compress_file)
    names = tar.getnames()
    for name in names:
        tar.extract(name, path=target_path)
    tar.close()


def rm(src_path: Union[str, Path], *args, **kwargs):
    """删除文件或目录
    :param src_path: str, 源文件或目录路径
    """
    check_path_is_exits(src_path)

    # 统一转换成Path处理
    if isinstance(src_path, str):
        src_path = Path(src_path)

    if src_path.is_file():
        src_path.unlink()
    else:
        shutil.rmtree(str(src_path), onerror=rm_readonly)

    logger.info(f"删除成功：{src_path}")


def rm_readonly(fn, tmp, info):
    """删除只读文件"""
    os.chmod(tmp, stat.S_IWRITE)
    if os.path.isfile(tmp):
        os.remove(tmp)
    elif os.path.isdir(tmp):
        shutil.rmtree(tmp)


def move_to_dir(
    src_path: Union[str, Path], dst_path: Union[str, Path], *args, **kwargs
):
    """移动文件
    :param src_path: str or Path, 源文件或目录路径
    :param dst_path: str or Path, 目标文件或目录路径
    """
    src_path = Path(src_path)
    dst_path = Path(dst_path)

    check_path_is_exits(src_path)
    if not dst_path.exists():
        dst_path.mkdir(parents=True)
    shutil.move(str(src_path), str(dst_path))


def copy_to_target(
    src_path: Union[str, Path], dst_path: Union[str, Path], *args, **kwargs
):
    """复制文件或目标到目标路径
    :param src_path: str, 源文件或目录路径，不支持正则表达式；
    :param dst_path: str, 目标字符串路径；
    """
    src_path = Path(src_path)
    dst_path = Path(dst_path)
    check_path_is_exits(src_path)

    if not dst_path.parent.exists():
        dst_path.parent.mkdir(parents=True)

    # 当源对象为文件时
    if src_path.is_file():
        shutil.copy2(str(src_path), str(dst_path))

    # 当源对象为目录时
    if src_path.is_dir():
        if dst_path.exists():
            # 将目录复制到已经存在的目录下
            shutil.copytree(
                str(src_path), str(dst_path / src_path.name), dirs_exist_ok=True
            )
        else:
            # 复制源目标生成指定的新目录
            shutil.copytree(str(src_path), str(dst_path), dirs_exist_ok=True)
            return


def copy_to_target_by_pattern(
    src_path: Union[str, Path],
    dst_path: Union[str, Path],
    recursion: bool = True,
    excludes: str = "",
):
    """通过表达式复制文件或目标到目标路径
    :param src_path: str, 源文件或目录路径，支持正则表达式；
    :param dst_path: str, 目标字符串路径；
    :param recursion: bool, 递归子目录；
    :param excludes: str, 排除表达式，排除单个目录如："x86"，同时排除多个目录用分号分隔如"x86;x64"；
    """
    src_path = str(src_path)
    dst_path = str(dst_path)
    # 两种情况，表达式与绝对路径
    if os.path.exists(src_path):
        # 绝对路径
        pattern = r".+"
    else:
        # 表达式
        last_sep_index = src_path.rfind(os.sep)
        pattern = src_path[last_sep_index + 1 :]
        src_path = src_path[:last_sep_index]
    # 排除项表达式
    if excludes:
        excludes = excludes.split(";")
    else:
        excludes = []

    # 遍历源目录
    for child in os.listdir(src_path):
        # 跳过子目录
        if not recursion and os.path.isdir(src_path + os.sep + child):
            continue

        # 判断是否为排除的项
        is_exclude = False
        for _p in excludes:
            if _p and re.match(_p, child, re.I):
                is_exclude = True
                break
        if is_exclude:
            continue

        # 判断是否为匹配的项
        if re.match(pattern, child, re.I):
            copy_to_target(src_path + os.sep + child, dst_path)


def read_file_stream(file_path: str, start_index, end_index):
    """读取文件流
    :param file_path: str, 文件路径
    :param start_index: int, 读取起始位置
    :param end_index: int, 读取结尾位置
    """
    check_path_is_exits(file_path, path_type="file")
    with open(file_path, mode="rb") as _file:
        _file.seek(start_index)
        return _file.read(end_index - start_index)


def read_file_raw_content(file_path: Union[str, Path], encoding=None) -> tuple:
    """读取文件内容
    :param file_path: 文件路径
    :param encoding: 文件字符编码，没有指定时会按照utf-8，GBK，GB2312，GB18030依次读取
    :return 返回文件类型与读取成功的文件编码
    """
    check_path_is_exits(file_path)
    file_path = Path(file_path)

    if encoding:
        encoding_list = [encoding]
    else:
        encoding_list = ["utf-8", "GBK", "GB2312", "GB18030"]

    error = None
    for encoding in encoding_list:
        try:
            with open(file_path, encoding=encoding) as _file:
                return _file.read(), encoding
        except Exception as e:
            error = e
    else:
        raise error


def read_file_content(
    file_path: Union[Path, str], encoding=None, _return=None, default=None
):
    """读取文件内容
    :param file_path: 文件路径
    :param encoding: 文件字符编码，没有指定时会按照utf-8，GBK，GB2312，GB18030依次读取
    :param _return: 为json时返回json格式数据，否则返回str
    :param default: 文件内容为空时返回的默认值
    :return 返回文件类型与读取成功的文件编码
    """
    if default is None:
        default = {}
    _raw_content, _ = read_file_raw_content(file_path, encoding)
    if _return and _return.lower() == "json":
        _raw_content = _raw_content.strip()
        if _raw_content:
            return json.loads(_raw_content)
        else:
            return default
    else:
        return _raw_content


def save_obj_to_file(obj, file_abs_path: str, exist_ok=True, encoding="utf-8"):
    """保存对象到文件
    :param obj: 待保存对象，仅限文件对象，不能是二进制对象
    :param file_abs_path: 保存路径
    :param exist_ok: 为True时覆盖已经存在的文件，为False时抛错
    :param encoding: 文件编码
    """
    if isinstance(obj, (dict, list, tuple)):
        return save_json_to_file(obj, file_abs_path, exist_ok, encoding)

    if isinstance(obj, bytes):
        obj = obj.decode(encoding)
    else:
        obj = str(obj)

    with open(file_abs_path, mode="w", encoding=encoding) as _file:
        return _file.write(obj)


def save_json_to_file(
    content: dict or list or tuple,
    file_abs_path: Union[Path, str],
    exist_ok=True,
    encoding="utf-8",
    ensure_ascii=False,
    indent=2,
    cls=None,
):
    """将字典、列表、元组保存到文件中
    :param content: 待保存对象，仅限dict,list,tuple
    :param file_abs_path: 保存路径
    :param exist_ok: 为True时覆盖已经存在的文件，为False时抛错
    :param encoding: 文件编码
    :param ensure_ascii:
    :param indent: int, 格式化JSON对象时缩进字符数量
    :param cls: json dumps 的转换类
    """
    file_abs_path = Path(file_abs_path)
    if not exist_ok:
        if file_abs_path.exists():
            raise FileExistsError(f"文件已经存在：{file_abs_path}")

    if not file_abs_path.parent.exists():
        file_abs_path.parent.mkdir(parents=True)

    with open(file_abs_path, mode="w", encoding=encoding) as _file:
        return json.dump(
            content, _file, ensure_ascii=ensure_ascii, indent=indent, cls=cls
        )


def get_newest_file(target: str, _type: str = "c") -> str:
    """获取目录下最新的文件
    :param target: str，目标目录；
    :param _type: str，类型，c按创建时间，m按修改时间，a按访问时间
    """
    if _type.lower() not in ("c", "m", "a"):
        raise ValueError(f"_type参数非法，正确的取值为：a,m,c")
    else:
        _method = getattr(os.path, f"get{_type}time")

    if os.path.isdir(target):
        file_list = os.listdir(target)
        file_list.sort(key=lambda _file: _method(target + os.sep + _file))
        return target + os.sep + file_list[-1]


def get_file_time(file_path: Union[Path, str], _type: str = "c", _return=None):
    """根据文件修改日期获取插件版本号"""
    file_path = Path(file_path)
    if _type.lower() not in ("c", "m", "a"):
        raise ValueError(f"_type参数非法，正确的取值为：a,m,c")
    else:
        _method = getattr(file_path.stat(), f"st_{_type}time")
        _time_obj = datetime.datetime.fromtimestamp(_method)
        if _return == datetime.datetime:
            return _time_obj
        elif _return == datetime.date:
            return _time_obj.date()
        elif _return == float:
            return _time_obj.timestamp()
        elif _return == int:
            return round(_time_obj.timestamp())
        else:
            return _time_obj.strftime("%Y-%m-%d %H:%M:%S")


def get_ini_config_object(config_path: Path):
    """获取config.ini配置文件对象"""
    __config = configparser.ConfigParser()
    __config.read(config_path, encoding="utf-8")
    return __config


def save_ini_config_object(config_path: Path, _object: configparser.ConfigParser):
    """保存配置文件到config.int中"""
    with open(config_path, mode="w", encoding="utf-8") as _file:
        _object.write(_file)



#!/usr/bin/env python
# coding:utf-8
"""单例模式父类
作者：dengqingyong
邮箱：yu12377@163.com
时间：2018/11/7 19:06
"""


class Singleton(object):
    """单例模式父类，继承此类可快速定义单例类"""

    # 已经实例化的对象类
    _instance = None

    # 单例模式：确保一个类只能实例化一次
    def __new__(cls, *args, **kwargs):
        if cls._instance is None:
            cls._instance = super(Singleton, cls).__new__(cls)
        return cls._instance



import sys
import logging
import logging.handlers
from typing import List, Union
from pathlib import Path


log_format = logging.Formatter(
    "[%(asctime)s] %(filename)s/%(lineno)s/%(funcName)s/%(levelname)s: %(message)s"
)

console_handler = logging.StreamHandler(stream=sys.stdout)
console_handler.setFormatter(log_format)
console_handler.setLevel(logging.INFO)

logger = logging.getLogger("DengUtils")
logger.addHandler(console_handler)
logger.setLevel(logging.INFO)


def clean_handler():
    for handler in logger.handlers:
        logger.removeHandler(handler)


def configure_logger(
    level: int,
    handlers: List[logging.Handler] = None,
    log_file: Union[Path, str] = None,
    append: bool = True,
):
    """配置日志处理器
    :param level: int, 指定日志级别
    :param handlers: List[Handler], 日志处理器列表
    :param log_file: 日志文件存储路径
    :param append: 是否为追加模式。当为覆盖模式时会清空已有的handler再添加新的
    """
    # 设置日志级别
    if level in (
        logging.NOTSET,
        logging.DEBUG,
        logging.INFO,
        logging.WARNING,
        logging.ERROR,
        logging.FATAL,
    ):
        logger.setLevel(level)
    else:
        raise ValueError(f"level参数非法：{level}")
    for handler in logger.handlers:
        handler.setLevel(level)

    # 非追加模式时，先清空所有已有的handler
    # 必须放置在设置已有Handler日志级别之后，因为已有的Handler可能被别处引用
    # 如果放置在最前面会导致在别处引用的handler日志级别无法受level参数控制
    if not append:
        clean_handler()

    # 往日志句柄中添加处理器
    if handlers:
        for handler in handlers:
            if isinstance(handler, logging.Handler):
                handler.setLevel(level)
                logger.addHandler(handler)
            else:
                raise TypeError(f"类型错误，预期为{logging.Handler}类型，实际为{type(handler)}类型")

    if log_file:
        log_file = Path(log_file)
        if not log_file.parent.exists():
            log_file.parent.mkdir(parents=True)

        file_handler = logging.handlers.TimedRotatingFileHandler(
            log_file, when="D", backupCount=15, encoding="utf-8"
        )
        file_handler.setFormatter(log_format)
        file_handler.setLevel(level)
        logger.addHandler(file_handler)

    logger.debug("日志配置完成")



#!/usr/bin/env python
# coding:utf-8
"""功能简要说明
作者：dengqingyong
邮箱：yu12377@163.com
时间：2018/1/12 下午6:20
"""
import uuid
import time
import random
import string
from typing import Tuple
from xpinyin import Pinyin

from .addressinfo import addr
from .bankinfo import bank_bin_list


def get_random_string(length: int = 32) -> str:
    """获取随机字符串"""
    return "".join(random.choices(string.ascii_letters + string.digits, k=length))


def get_uuid(length: int = 32) -> str:
    _uuid36 = str(uuid.uuid1())
    _uuid32 = _uuid36.replace("-", "")
    if length == 36:
        return _uuid36
    elif length == 32:
        return _uuid32
    else:
        return _uuid32[:length]


def get_pinyin(name: str = "", joint_mark="") -> Tuple[str, str]:
    """获取拼音
    :param name: str, 需要转换拼音的汉字
    :param joint_mark: str, 拼音连接符号
    :return 返回拼音与源汉字的元组
    """
    if not name:
        name = get_name()

    p = Pinyin()
    username = p.get_pinyin(name, joint_mark)
    return username, name


def get_name(gender=""):
    """获取中国人惯用姓名"""
    firstnames = """
        赵钱孙李，周吴郑王。
        冯陈褚卫，蒋沈韩杨。
        朱秦尤许，何吕施张。
        孔曹严华，金魏陶姜。
        戚谢邹喻，柏水窦章。
        云苏潘葛，奚范彭郎。
        鲁韦昌马，苗凤花方。
        俞任袁柳，酆鲍史唐。
        费廉岑薛，雷贺倪汤。
        滕殷罗毕，郝邬安常。
        乐于时傅，皮卞齐康。
        伍余元卜，顾孟平黄。
        和穆萧尹，姚邵湛汪。
        祁毛禹狄，米贝明臧。
        计伏成戴，谈宋茅庞。
        熊纪舒屈，项祝董梁。
        杜阮蓝闵，席季麻强。
        贾路娄危，江童颜郭。
        梅盛林刁，钟徐邱骆。
        高夏蔡田，樊胡凌霍。
        虞万支柯，昝管卢莫。
        经房裘缪，干解应宗。
        丁宣贲邓，郁单杭洪。"""

    firstnames = (
        firstnames.replace("，", "")
        .replace("。", "")
        .replace("\n", "")
        .replace(" ", "")
    )
    firstname = random.choice(firstnames)

    secondname_boy = """
    澄邈、德泽、海超、海阳、海荣、海逸、海昌、瀚钰、瀚文、涵亮、涵煦、涵蓄、涵衍、浩皛、浩波、浩博、浩初、浩宕、浩歌、浩广、浩邈、浩气、
    浩思、浩言、鸿宝、鸿波、鸿博、鸿才、鸿畅、鸿畴、鸿达、鸿德、鸿飞、鸿风、鸿福、鸿光、鸿晖、鸿朗、鸿文、鸿轩、鸿煊、鸿骞、鸿远、鸿云、
    鸿哲、鸿祯、鸿志、鸿卓、嘉澍、光济、澎湃、彭泽、鹏池、鹏海、浦和、浦泽、瑞渊、越泽、博耘、德运、辰宇、辰皓、辰钊、辰铭、辰锟、辰阳、
    辰韦、辰良、辰沛、晨轩、晨涛、晨濡、晨潍、鸿振、吉星、铭晨、起运、运凡、运凯、运鹏、运浩、运诚、运良、运鸿、运锋、运盛、运升、运杰、
    运珧、运骏、运凯、运乾、维运、运晟、运莱、运华、耘豪、星爵、星腾、星睿、星泽、星鹏、星然、震轩、震博、康震、震博、振强、振博、振华、
    振锐、振凯、振海、振国、振平、昂然、昂雄、昂杰、昂熙、昌勋、昌盛、昌淼、昌茂、昌黎、昌燎、昌翰、晨朗、德明、德昌、德曜、范明、飞昂、
    高旻、晗日、昊然、昊天、昊苍、昊英、昊宇、昊嘉、昊明、昊伟、昊硕、昊磊、昊东、鸿晖、鸿朗、华晖、金鹏、晋鹏、敬曦、景明、景天、景浩、
    俊晖、君昊、昆琦、昆鹏、昆纬、昆宇、昆锐、昆卉、昆峰、昆颉、昆谊、昆皓、昆鹏、昆明、昆杰、昆雄、昆纶、鹏涛、鹏煊、曦晨、曦之、新曦、
    旭彬、旭尧、旭鹏、旭东、旭炎、炫明、宣朗、学智、轩昂、彦昌、曜坤、曜栋、曜文、曜曦、曜灿、曜瑞、智伟、智杰、智刚、智阳、昌勋、昌盛、
    昌茂、昌黎、昌燎、昌翰、晨朗、昂然、昂雄、昂杰、昂熙、范明、飞昂、高朗、高旻、德明、德昌、德曜、智伟、智杰、智刚、智阳、瀚彭、旭炎、
    宣朗、学智、昊然、昊天、昊苍、昊英、昊宇、昊嘉、昊明、昊伟、鸿朗、华晖、金鹏、晋鹏、敬曦、景明、景天、景浩、景行、景中、景逸、景彰、
    昆鹏、昆明、昆杰、昆雄、昆纶、鹏涛、鹏煊、景平、俊晖、君昊、昆琦、昆鹏、昆纬、昆宇、昆锐、昆卉、昆峰、昆颉、昆谊、轩昂、彦昌、曜坤、
    曜文、曜曦、曜灿、曜瑞、曦晨、曦之、新曦、鑫鹏、旭彬、旭尧、旭鹏、旭东、浩涆、浩瀚、浩慨、浩阔、鸿熙、鸿羲、鸿禧、鸿信、泽洋、泽雨、
    哲瀚、胤运、佑运、允晨、运恒、运发、云天、耘志、耘涛、振荣、振翱、中震、子辰、晗昱、瀚玥、瀚昂、瀚彭、景行、景中、景逸、景彰、绍晖、
    文景、曦哲、永昌、子昂、智宇、智晖、晗日、晗昱、瀚玥、瀚昂、昊硕、昊磊、昊东、鸿晖、绍晖、文昂、文景、曦哲、永昌、子昂、智宇、智晖、
    浩然、鸿运、辰龙、运珹、振宇、高朗、景平、鑫鹏、昌淼、炫明、昆皓、曜栋、文昂"""

    secondname_grid = """
    恨桃、依秋、依波、香巧、紫萱、涵易、忆之、幻巧、水风、安寒、白亦、惜玉、碧春、怜雪、听南、念蕾、紫夏、凌旋、芷梦、凌寒、梦竹、千凡、
    采波、元冬、思菱、平卉、笑柳、雪卉、南蓉、谷梦、巧兰、绿蝶、飞荷、平安、芷荷、怀瑶、慕易、若芹、紫安、曼冬、寻巧、寄波、尔槐、以旋、
    初夏、依丝、怜南、傲菡、谷蕊、笑槐、飞兰、笑卉、迎荷、元冬、痴安、妙绿、觅雪、寒安、沛凝、白容、乐蓉、映安、依云、映冬、凡雁、梦秋、
    梦凡、秋巧、若云、元容、怀蕾、灵寒、天薇、翠安、乐琴、宛南、怀蕊、白风、访波、亦凝、易绿、夜南、曼凡、亦巧、青易、冰真、白萱、友安、
    海之、小蕊、又琴、天风、若松、盼菡、秋荷、香彤、语梦、惜蕊、迎彤、沛白、雁山、易蓉、雪晴、诗珊、春冬、又绿、冰绿、半梅、笑容、沛凝、
    映秋、盼烟、晓凡、涵雁、问凝、冬萱、晓山、雁蓉、梦蕊、山菡、南莲、飞双、凝丝、思萱、怀梦、雨梅、冷霜、向松、迎丝、迎梅、雅彤、香薇、
    以山、碧萱、寒云、向南、书雁、怀薇、思菱、忆文、翠巧、怀山、若山、向秋、凡白、绮烟、从蕾、天曼、又亦、从安、绮彤、之玉、凡梅、依琴、
    沛槐、又槐、元绿、安珊、夏之、易槐、宛亦、白翠、丹云、问寒、易文、傲易、青旋、思真、雨珍、幻丝、代梅、盼曼、妙之、半双、若翠、初兰、
    惜萍、初之、宛丝、寄南、小萍、静珊、千风、天蓉、雅青、寄文、涵菱、香波、青亦、元菱、翠彤、春海、惜珊、向薇、冬灵、惜芹、凌青、谷芹、
    雁桃、映雁、书兰、盼香、向山、寄风、访烟、绮晴、映之、醉波、幻莲、谷冬、傲柔、寄容、以珊、紫雪、芷容、书琴、寻桃、涵阳、怀寒、易云、
    代秋、惜梦、尔烟、谷槐、怀莲、夜山、芷卉、向彤、新巧、语海、灵珊、凝丹、小蕾、迎夏、慕卉、飞珍、冰夏、亦竹、飞莲、海白、元蝶、春蕾、
    怀绿、尔容、小玉、幼南、凡梦、碧菡、初晴、宛秋、傲旋、新之、凡儿、夏真、静枫、痴柏、恨蕊、乐双、念薇、靖雁、寄松、丹蝶、元瑶、冰蝶、
    念波、迎松、海瑶、乐萱、凌兰、曼岚、若枫、傲薇、凡灵、乐蕊、秋灵、谷槐、觅云、寻春、恨山、从寒、忆香、觅波、静曼、青寒、笑天、涵蕾、
    元柏、代萱、紫真、千青、雪珍、寄琴、绿蕊、醉柳、诗翠、念瑶、孤风、曼彤、怀曼、香巧、采蓝、芷天、尔曼、巧蕊"""

    # 随机获取男名与女名
    grid_name = random.choice(
        secondname_grid.replace("\n", "").replace(" ", "").split("、")
    )
    boy_name = random.choice(
        secondname_boy.replace("\n", "").replace(" ", "").split("、")
    )

    # 将两个字的名字随机转换成双字或单字
    if random.randint(1, 3) == 1:
        grid_name = random.choice(grid_name)
        boy_name = random.choice(boy_name)

    # 男名or女名
    if gender.lower() in ("girl", "woman"):
        secondname = grid_name
    elif gender.lower() in ("boy", "man"):
        secondname = boy_name
    elif isinstance(gender, int):
        if gender % 2 == 0:
            secondname = grid_name
        else:
            secondname = boy_name
    else:
        secondname = random.choice([boy_name, grid_name])

    return "{}{}".format(firstname, secondname)


def check_idcards(idcards):
    """函数功能：
    校验模式：1. 校验中国大陆身份证号码的第18位是否正确；
    计算模式：2. 传入前17位，计算第18位校验码返回；
    返回为空时代码校验不通过；
    返回有值时代码校验通过，或是计算模式；
    """
    check_code = ""
    # 统一转换成字符串类型
    try:
        idcards = str(idcards)
    except TypeError as e:
        pass

    # 判断传入的身份证号码长度，如果为18位时顺便截取校验码
    if len(idcards) == 18:
        check_code = idcards[-1:]
        idcards = idcards[:-1]
    elif len(idcards) == 17:
        pass
    else:
        print("输入的身份证号码【{}】位数有误，必须是18位！".format(idcards))
        return check_code

    # 至此已经被转成17位
    # 十七位数字本体码权重
    weight = [7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2]
    # 对应校验码字符值
    validate = ["1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2"]

    _sum = 0
    _mode = 0
    for i in range(0, 17):
        try:
            _sum = _sum + int(idcards[i]) * weight[i]
        except TypeError as e:
            print("输入的身份证号码【{}】前17位必须是数字！".format(idcards))
    _mode = _sum % 11
    # 计算模式
    if check_code == "":
        return validate[_mode]
    # 校验模式——校验通过
    elif check_code == validate[_mode]:
        return check_code
    # 校验模式——校验不通过
    else:
        return ""


def get_idcards(sex=0):
    """获取中国大陆18位身份证号码"""
    # part1: 随机获取地区编码
    area_code = random.choice(addr)[0]

    # part2: 随机获取1949/10/01到当天的一个日期
    # 日期起始时间: 1949/10/01 00:00:00
    start_timestamp = -639129600
    # 日期结束时间：当天
    end_timestamp = round(time.time())
    birthdate_timestamp = random.randrange(start_timestamp, end_timestamp, 86400)
    birthdate = time.strftime("%Y%m%d", time.localtime(birthdate_timestamp))

    # part3: 随机获取两位顺序码
    sequence_code = random.randrange(11, 99)

    # part4：性别编码：奇怪代表男性，偶数代码女性
    if sex == 0:
        sex_code = random.randrange(1, 10)
    elif sex % 2 == 1:
        sex_code = random.randrange(1, 10, 2)
    else:
        sex_code = random.randrange(2, 10, 2)

    # part5: 计算校验码
    check_code = check_idcards(
        "{}{}{}{}".format(area_code, birthdate, sequence_code, sex_code)
    )

    return "{}{}{}{}{}".format(
        area_code, birthdate, sequence_code, sex_code, check_code
    )


def get_mobile_no():
    """生成随机手机号码"""
    phone_head = [
        130,
        131,
        132,
        133,
        134,
        135,
        136,
        137,
        138,
        139,
        150,
        151,
        152,
        155,
        158,
        170,
        171,
        172,
        173,
        174,
        175,
        176,
        177,
        178,
        179,
        181,
        186,
        187,
        188,
        189,
    ]
    phone_no = str(random.choice(phone_head)) + "".join(
        random.choice("0123456789") for i in range(8)
    )
    return phone_no


def get_phone_serial_no():
    """生成手机串号"""
    serial_no = "".join(random.choice(string.ascii_uppercase) for i in range(4))
    serial_no += "-" + "".join(
        random.choice(string.ascii_uppercase) for i in range(4)
    )
    serial_no += "-" + "".join(random.choice(string.digits) for i in range(5))
    return serial_no


def get_bank_no(num=1, bank=None, bank_name=None, ftype=None, length=None, first=None):
    """生成银联卡卡号
    :param bank: 银行简称，大写字母，如工行ICBC，建行CCB，农行ABC等——非必填，默认随机
    :param ftype: 卡片类型，储蓄卡DC，信用卡CC——非必填，默认随机
    :param length: 卡号长度，信用卡基本上都是16位，储蓄卡通常16至19位，最长19位，但偶尔有低有16位的——非必填，默认随机
    :param num: 一次生成的卡号数量——非必填，默认1
    :param first: 为True时返回一个对象，非True时返回列表
    """

    if length and (int(length) < 16 or int(length) > 19):
        raise ValueError("银联卡号通常是16到19数字，请检查输入的length参数")

    # 获取参数获取bin码
    bin_list = get_bank_bin(
        num=num, bank=bank, bank_name=bank_name, ftype=ftype, length=length
    )

    for bin in bin_list:
        __get_bank_no(bin_obj=bin)
    if first:
        return bin_list[0]
    else:
        return bin_list


def __get_bank_no(bin_obj):
    if not bin_obj["bin"].isdigit():
        raise ValueError("银行卡BIN应该为6位数字")

    # 中间数字长度=长度 - 6位bin长度 - 末位校验码
    part2_length = int(bin_obj["length"]) - len(bin_obj["bin"]) - 1
    bank_part2 = ""
    for i in range(part2_length):
        bank_part2 += str(random.randint(0, 9))

    # 计算末尾校验码
    bank_no = bin_obj["bin"] + bank_part2
    fsum = 0
    _index = 0
    while True:
        _index -= 1
        try:
            _value = int(bank_no[_index])
        except IndexError as e:
            break
        # 判断位置
        if abs(_index) % 2 == 1:
            # 倒数：奇数位时*2
            _value *= 2
            if _value >= 10:
                # 十位数时：取个位数字与十位数字相加
                fsum += _value - 9
            else:
                # 个位数时：直接采用
                fsum += _value
        else:
            # 倒数：偶数位时直接相加
            fsum += _value
    # 拼接末尾校验码
    if fsum % 10 == 0:
        check_no = 0
    else:
        check_no = 10 - fsum % 10
    bank_no += str(check_no)
    bin_obj["no"] = bank_no
    return bin_obj


def get_bank_bin(
    num: int = 1,
    bank: str = None,
    bank_name: str = None,
    ftype: str = None,
    length: str = None,
):
    """获取银行卡bin码"""
    if length:
        length = str(length)
    bin_list = []
    # 根据条件过滤
    for _bin in bank_bin_list:
        if bank and _bin["bank"] != bank:
            continue
        if bank_name and not _bin["name"].startswith(bank_name):
            continue
        if ftype and _bin["type"] != ftype:
            continue
        if length and _bin["length"] != length:
            continue
        bin_list.append(_bin)
    if not bin_list:
        if ftype == "CC" and int(length) > 16:
            raise ValueError("找不到对应的bin码，信用卡通常是16位，请检查输入的length参数")
        raise ValueError("找不到对应的bin码，请检查输入是否正确！")
    if num <= 0:
        return bin_list
    else:
        return random.choices(bin_list, k=num)
